using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using InternshipSystem.Core;
using InternshipSystem.Core.Entity.Internship;
using InternshipSystem.Core.UglyOrmArtifacts;
using Microsoft.EntityFrameworkCore;

namespace InternshipSystem.Repository
{
    public class DatabaseFiller
    {
        public DatabaseFiller(InternshipDbContext context)
        {
            Context = context;
        }

        public InternshipDbContext Context { get; }

        public async Task FillCompanies()
        {
            var companies = new List<Company>
            {
                new Company
                {
                    Id = 1,
                    Name = "Intel",
                    SiteAddress = new Uri("https://www.intel.com/content/www/us/en/jobs/locations/poland.html"),
                    Nip = "9570752316",
                    Range = RangeOfActivity.International,
                    Branches = new List<BranchOffice>
                    {
                        new BranchOffice
                        {
                            Address =  new BranchAddress 
                            {
                                City = "Gdańsk",
                                Street = "ul. Słowackiego",
                                Building = "173",
                                PostalCode = "80-298",
                                Country = "Poland",
                            }
                        },
                        new BranchOffice
                        {
                            Address =  new BranchAddress 
                            {
                                City = "Gdańsk",
                                Street = "Jana z Kolna",
                                Building = "11",
                                PostalCode = "80-001",
                                Country = "Poland",
                            }
                        },
                        new BranchOffice
                        {
                            Address =  new BranchAddress 
                            {
                                City = "Boston",
                                Street = "State St",
                                Building = "53",
                                PostalCode = "MA 02109",
                                Country = "Stany Zjednoczone",
                            }
                        }
                    }
                },
                new Company
                {
                    Id = 2,
                    Name = "Asseco Poland",
                    SiteAddress = new Uri("http://pl.asseco.com"),
                    Nip = "5842068320",
                    Range = RangeOfActivity.National,
                    Branches = new List<BranchOffice>
                    {
                        new BranchOffice
                        {
                            Address =  new BranchAddress 
                            {
                                City = "Gdańsk",
                                Street = "ul. Podolska",
                                Building = "21",
                                PostalCode = "81-321",
                                Country = "Poland"
                            }
                        },
                        new BranchOffice
                        {
                            Address =  new BranchAddress 
                            {
                                City = "Wrocław",
                                Street = "Traugutta",
                                Building = "1/7",
                                PostalCode = "50-449",
                                Country = "Poland"
                            }
                        }
                    }
                }
            };
            await Context.Companies.AddRangeAsync(companies);
            await Context.SaveChangesAsync();
        }
        
        public async Task FillEditions()
        {
            var editions = new List<Edition>
            {
                new Edition
                {
                    EditionStart = new DateTime(2000, 5, 10),
                    EditionFinish = new DateTime(2000, 11, 10),
                    ReportingStart = new DateTime(2000, 9, 30),
                    AvailableSubjects = new List<EditionSubject>
                    {
                        new EditionSubject
                        {
                            Subject = new InternshipSubject
                            {
                                Description = "Modelowanie baz danych"
                            }
                        },
                        new EditionSubject
                        {
                            Subject = new InternshipSubject 
                            {
                                Description = "Oprogramowywanie kart graficznych"
                            }
                        },
                        new EditionSubject
                        {
                            Subject = new InternshipSubject
                            {
                                Description = "Projektowanie UI"
                            }
                        }
                    },
                    Course = new Course
                    {
                        Name = "Informatyka",
                    },
                    Internships = new List<Internship>(),
                }
            };
            
            var edition = editions.First();
            edition.Internships.AddRange(
                new List<Internship> 
                {
                    new Internship
                    {
                        Student = new Student
                        {
                            FirstName = "Jan",
                            LastName = "Kowalski",
                            AlbumNumber = 123456,
                            Email = "s123456@student.pg.edu.pl",
                            Semester = 4,
                        },
                        InternshipRegistration = new InternshipRegistration
                        {
                            Company = Context.Companies.First(c => c.Id.Equals(1)), //Intel
                            Type =  new InternshipType
                            {
                                Type = "UOP"
                            },
                            Start = new DateTime(2000, 7, 1),
                            End = new DateTime(2000, 8, 30),
                            State = DocumentState.Submitted,
                            BranchAddress =
                                Context.Companies
                                    .Include(c => c.Branches)
                                    .First(c => c.Id.Equals(1))    
                                    .Branches
                                    .First(),
                        },
                        InternshipProgram = new InternshipProgram
                        {
                            Mentor = new Mentor 
                            {
                                FirstName = "Horacy",
                                LastName = "Wościcki",
                                Email = "howos@intel.com",
                                PhoneNumber = "605-555-555",
                            },
                            ChosenSubjects = new List<ProgramSubject>
                            {
                                new ProgramSubject
                                {
                                    Subject = edition.AvailableSubjects
                                        .First(s => s.Subject.Description.Equals("Modelowanie baz danych"))
                                        .Subject
                                },
                                new ProgramSubject
                                {
                                    Subject = edition.AvailableSubjects
                                        .First(s => s.Subject.Description.Equals("Projektowanie UI"))
                                        .Subject
                                }
                            }
                        },
                    },
                    new Internship
                    {
                        Student = new Student
                        {
                            FirstName = "Adam",
                            LastName = "Kołek",
                            AlbumNumber = 102137,
                            Email = "s102137@student.pg.edu.pl",
                            Semester = 6,
                        },
                        InternshipRegistration = new InternshipRegistration
                        {
                            Company = Context.Companies.First(c => c.Id.Equals(2)), //Asseco
                            Type = new InternshipType
                            {
                                Type = "UZ"
                            },
                            Start = new DateTime(2000, 7, 1),
                            End = new DateTime(2000, 8, 30),
                            State = DocumentState.Submitted,
                            BranchAddress =
                                Context.Companies
                                    .Include(c => c.Branches)
                                    .First(c => c.Id.Equals(2))
                                    .Branches
                                    .First(),
                        },
                        InternshipProgram = new InternshipProgram
                        {
                            Mentor = new Mentor 
                            {
                                FirstName = "Henryk",
                                LastName = "Polaciński",
                                Email = "hepol@asseco.pl",
                                PhoneNumber = "555-525-545",
                            },
                            ChosenSubjects = new List<ProgramSubject>
                            {
                                new ProgramSubject
                                {
                                    Subject = edition.AvailableSubjects
                                        .First(s => s.Subject.Description.Equals("Oprogramowywanie kart graficznych"))
                                        .Subject
                                }
                            },
                        },
                    },
                });
            await Context.Editions.AddRangeAsync(editions);
            await Context.SaveChangesAsync();
        }
        
        // new InternshipType
        // {
        //     Type = "FreeInternship",
        //     Description = "Praktyka bezpłatna",
        // },
        // new InternshipType
        // {
        //     Type = "GraduateInternship"
        // },
        // new InternshipType
        // {
        //     Type = "FreeApprenticeship"
        // },
        // new InternshipType
        // {
        //     Type = "PaidApprenticeship",
        //     Description = "np. przemysłowy"
        // },
        // new InternshipType
        // {
        //     Type = "ForeignInternship",
        //     Description = "np. IAESTE, ERASMUS"
        // },
        // new InternshipType
        // {
        //     Type = "UOP"
        // },
        // new InternshipType
        // {
        //     Type = "UD"
        // },
        // new InternshipType
        // {
        //     Type = "UZ"
        // },
        // new InternshipType
        // {
        //     Type = "Other",
        //     Description = "Należy wprowadzić samodzielnie"
        // }
    }
}